import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import processing.sound.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class misplaced extends PApplet {



PVector center;
ArrayList<Island> islands;

SoundFile file;
PImage background_image;

float luminosity;
float island_lum;
float hostile_lum;

int PROGRAM_STATE;

public void settings () {
  
  size(1200, 800, P2D);
  pixelDensity(2);
  file = new SoundFile(this, "waves.mp3");
  jellySound = new SoundFile(this, "electricshock.mp3");
  sharkSound = new SoundFile(this, "sharkSound.mp3");
  playerDeathSound = new SoundFile(this, "deathSound.mp3");
  destroyPalmtree = new SoundFile(this, "destroyPalmtree.mp3");
  background_image = loadImage("seamless.jpg");
  center = new PVector(width/2, height/2);
  
  PROGRAM_STATE = 0;
  
  create_background();
  createResources();
  
  islands = generate_islands(30, 1200.0f);
  generateHostiles(90, 1500.0f);  
  generateBottles(20, 1200.0f);
}

public void setup () {
  frameRate(60);
  imageMode(CENTER);
  file.amp(0.3f);
  file.loop();
}

public void text_render () {
  textSize(128);
  text("Misplaced", center.x, center.y - 200);
  textSize(64);
  text("Press E to begin playing", center.x, center.y - 100);
  textSize(48);
  text("Controls:", center.x, center.y + 100);
  text("Press E to pick up bottles or wood", center.x, center.y + 140);
  text("Press F to punch trees or sharks", center.x, center.y + 180);
  text("Press R to reload or regenerate", center.x, center.y + 220);
}

public void death_text () {
  textSize(256);
  text("You Died", center.x, center.y - 100);
  textSize(64);
  text("Press R to reload or regenerate", center.x, center.y + 100);
}

public void win_text () {
  textSize(256);
  text("You Won!", center.x, center.y - 100);
  textSize(64);
  text("Press R to reload or regenerate", center.x, center.y + 100);
  text("Press Q to exit", center.x, center.y + 200);
}

public void draw () {
  luminosity = 1;
  island_lum = 1;
  hostile_lum = 1;
  switch(PROGRAM_STATE) {
    case 0: {
      background(0, 0, 255);
      renderHostiles();
      handle_background();
      render_islandWater(islands);
      render_islandBackground(islands);
      render_islands(islands);
      textAlign(CENTER);
      // Black Text
      fill(0);
      pushMatrix();
        translate(-1, -1);
        text_render();
        translate(2, 0);
        text_render();
        translate(0, 2);
        text_render();
        translate(-2, 0);
        text_render();
      popMatrix();
      // White Text
      fill(255);
      text_render();
      break;
    }
    case 1: {
      luminosity = sin(frameCount / 999.0f);
      island_lum = map(luminosity, -1, 1, 0.2f, 1);
      hostile_lum = map(luminosity, -1, 1, 1, 0.3f);
      background(0, 0, 255.0f * luminosity);
      renderHostiles();
      handle_background();
      render_islandWater(islands);
      render_bottles(bottleArray);
      render_islandBackground(islands);
      render_islands(islands);
      player.update();
      player.render();
      player.attacked();
      player.playerWin();
      renderResources();
      noFill();
      stroke(255);
      textSize(12);
      text(frameRate, width-50, height/3);
      text(player.getPosition().x, width-50, height/3+30);
      text(player.getPosition().y, width-50, height/3+60);
      player.displayInventory(1000.0f, 600.0f);
      break;
    }
    case 2: {
      background(0, 0, 255);
      renderHostiles();
      handle_background();
      render_islandWater(islands);
      render_islandBackground(islands);
      render_islands(islands);
      textAlign(CENTER);
      // Black Text
      fill(0);
      pushMatrix();
        translate(-1, -1);
        death_text();
        translate(2, 0);
        death_text();
        translate(0, 2);
        death_text();
        translate(-2, 0);
        death_text();
      popMatrix();
      // White Text
      fill(255, 50, 50);
      death_text();
      break;
    }
    case 3: {
      background(0, 0, 255);
      renderHostiles();
      handle_background();
      render_islandWater(islands);
      render_islandBackground(islands);
      render_islands(islands);
      textAlign(CENTER);
      // Black Text
      fill(0);
      pushMatrix();
        translate(-1, -1);
        win_text();
        translate(2, 0);
        win_text();
        translate(0, 2);
        win_text();
        translate(-2, 0);
        win_text();
      popMatrix();
      fill(255, 255, 0);
      win_text();
      break;
    }
  }  
}
SoundFile pickup;
Resource bananas, potatoes, wood, bottles;
ArrayList<ResourceInstance> bottleArray;

public void createResources () {
  bananas = new Resource("Banana", "https://i.pinimg.com/originals/4d/63/40/4d6340bb586909c97ad0aa717868194d.png", "png");
  potatoes = new Resource("Potatoes");
  wood = new Resource("Wood", "logsNoBackground copy.png", "png");
  bottles = new Resource("Bottle", "waterBottleNoBackground.png", "png");
  bottleArray = new ArrayList<ResourceInstance>();
  
  resources.add(bananas);
  resources.add(potatoes);
  resources.add(wood);
  resources.add(bottles);
  pickup = new SoundFile(this, "collect.mp3");
}

class PotatoInstance extends ResourceInstance {
  PotatoInstance (PVector pos, float radius) {
    super(potatoes, pos, radius);
  }
  public float getSize () {
    return 30;
  }
}

class BananaInstance extends ResourceInstance {
  private float actual_size;
  BananaInstance (PVector pos, float radius) {
    super(bananas, pos, radius);
    this.actual_size = random(15, 45);
  }
  public float getSize () {
    return this.actual_size;
  }
}

class WoodInstance extends ResourceInstance {
  WoodInstance (PVector pos, float radius) {
    super(wood, pos, radius);
  }
  public float getSize () {
    return 35;
  }
}

class BottleInstance extends ResourceInstance {

  BottleInstance(PVector pos, float radius) {
    super(bottles, pos, radius);
    bottles.loadedImage.resize(15, 0);
  }

  public float getSize() {
    return 35;
  }

  public void render() {
    float floatVal = sin(millis()/600.0f);
    PImage img = bottles.getImage();
    PVector pos = this.getRelativePos();
    float size = this.getSize();
    if (img != null) {
      image(img, pos.x, pos.y + floatVal*10, size, size);
    } else {
      fill(255);
      stroke(0);
      circle(pos.x, pos.y + floatVal*50.0f, size);
    }
  }
}

public void generateBottles(int amount, float radius) {
  for (int i = 0; i < amount; i++){
    BottleInstance newBottle = new BottleInstance(new PVector(0, 0), radius);
    bottleArray.add(newBottle);
 
    // remove bottle from intances, need to render separatley
    bottles.instances.remove(newBottle);
  }
}

public void render_bottles(ArrayList<ResourceInstance> bottleArray){
  for (ResourceInstance bottle: bottleArray){
    if (bottle.visible()){
      bottle.render();
    }
  }
}
public static final int SHARK = 0;
public static final int JELLY = 1;

SoundFile jellySound;
SoundFile sharkSound;

PImage shark_sheet;
PImage shark_frame[] = new PImage[7];

PImage jelly_sheet;
PImage jelly_frame[] = new PImage[9];

public ArrayList<Hostile> hostiles;

public class Hostile extends Plottable {

  private int type;

  private float angle;
  private float angle_start;
  private float target_angle;

  private float x_movement = 1;
  private SoundFile attackSound;

  Hostile (int type, PVector pos) {
    super(pos);
    this.type = type;
    this.angle = random(0, TWO_PI);
    this.angle_start = millis();
    this.target_angle = random(0, TWO_PI);

    switch (this.type) {
    case SHARK: 
      {
        this.attackSound = sharkSound;
      } 
      break;
    case JELLY: 
      {
        this.attackSound = jellySound;
      } 
      break;
    }
  }

  Hostile (int type, PVector pos, float radius) {
    super(pos, radius);
    this.type = type;
    this.angle = random(0, TWO_PI);
    this.angle_start = millis();
    this.target_angle = random(0, TWO_PI);
    switch (this.type) {
    case SHARK: 
      {
        this.attackSound = sharkSound;
      } 
      break;
    case JELLY: 
      {
        this.attackSound = jellySound;
      } 
      break;
    }
  }

  public float getSize () {
    return 32;
  }
  
  public void playSound(){
    this.attackSound.play();
  }

  private void newAngle () {
    this.angle = target_angle;
    this.angle_start = PApplet.parseFloat(millis());
    this.target_angle = random(0, TWO_PI);
  }

  private PVector lerpAngle(float i) {
    PVector va = PVector.fromAngle(this.angle);
    PVector vb = PVector.fromAngle(this.target_angle);
    return va.lerp(vb, i).normalize();
  }

  public void update () {
    float delta = (PApplet.parseFloat(millis()) - angle_start) / 1000.0f;
    if (delta > 5) newAngle();
    PVector movement = this.lerpAngle(delta).mult(2).mult(hostile_lum);
    this.move(movement);
    this.x_movement = movement.x;
    if (this.getPosition().dist(new PVector(0, 0)) > 2000) {
      super.pos = new PVector(0, 0);
    }
  } 

  public void render () {
    PVector loc = this.getRelativePos();
    switch (this.type) {
    case SHARK: 
      {
        int frame = PApplet.parseInt(frameCount * 0.2f) % 7;
        if (this.x_movement > 0) {
          image(shark_frame[frame], loc.x, loc.y, this.getSize(), this.getSize());
        } else {
          pushMatrix();
          scale(-1, 1);
          image(shark_frame[frame], -loc.x, loc.y, this.getSize(), this.getSize());
          popMatrix();
        }
        break;
      } 
    case JELLY: 
      {
        int frame = PApplet.parseInt(frameCount * 0.2f) % 9;
        if (this.x_movement > 0) {
          image(jelly_frame[frame], loc.x, loc.y, this.getSize(), this.getSize());
        } else {
          pushMatrix();
          scale(-1, 1);
          image(jelly_frame[frame], -loc.x, loc.y, this.getSize(), this.getSize());
          popMatrix();
        }
        break;
      }
    }
  }
}

public void renderHostiles () {
  for (Hostile hostile : hostiles) {
    hostile.update();
    if (hostile.visible()) {
      hostile.render();
    }
  }
}

public void generateHostiles (int count, float radius) {
  hostiles = new ArrayList<Hostile>();
  shark_sheet = loadImage("shark_sprite.png", "png");
  jelly_sheet = loadImage("jelly_sprite.png", "png");
  for (int i = 0; i < 7; i++) {
    shark_frame[i] = shark_sheet.get((i%5) * 128, PApplet.parseInt(i/5)*128, 128, 128);
  }
  for (int i = 0; i < 9; i++) {
    jelly_frame[i] = jelly_sheet.get(1 + (i%5) * 129, PApplet.parseInt(i/5)*129, 128, 128);
  }
  for (int i = 0; i < count; i++) {
    if (random(1.0f) > 0.5f) {
      hostiles.add(new Hostile(SHARK, center, radius));
    } else {
      hostiles.add(new Hostile(JELLY, center, radius));
    }
  }
}
public class Island extends Plottable {
  ArrayList<Palmtree> palmtrees;
  ArrayList<WoodInstance> woodLogs;

  private float size;
  private int numPalmtrees;
  public float beachSize;
  public float inlandSize;

  Island(PVector pos, float radius) {
    super(pos, radius);
    this.inlandSize = random(80, 150);
    this.beachSize = random(this.inlandSize/6, this.inlandSize/4);
    this.size = inlandSize + beachSize;
    palmtrees = new ArrayList<Palmtree>();
    this.numPalmtrees = PApplet.parseInt(random(3, 7));
  }

  // access 
  public float getSize() {
    return this.size;
  }
  public int getNumPalmtrees() {
    return this.numPalmtrees;
  }

  public void addLog() {
    WoodInstance woodLog = new WoodInstance(this.getPosition(), 5.0f);
    woodLogs.add(woodLog);
  }

  // fill island with palmtrees
  public void insertPalmtrees() {
    for (int i = 0; i < numPalmtrees; i++) {
      Palmtree newPalmtree = new Palmtree(this.getPosition(), this.getSize()/2 - 5); // palmtree radius=5
      newPalmtree.palmtreeImage = loadImage("palmtreeNoBackground-min.png");
      newPalmtree.palmtreeImage.resize(40, 0);
      palmtrees.add(newPalmtree);
    }
  }

  // Render island
  public void render() {
    noStroke();
    PVector pos = this.getRelativePos();
    fill(245 * island_lum, 235 * island_lum, 214 * island_lum);
    circle(pos.x, pos.y, this.inlandSize);
  }

  // render teal water color under island and beach
  public void renderIslandWater() {
    noStroke();
    PVector pos = this.getRelativePos();
    fill(93 * island_lum, 206 * island_lum, 231 * island_lum);
    float wave = sin(PApplet.parseFloat(millis()) / 1500.0f);
    circle(pos.x, pos.y, this.getSize() + abs(wave)*20);
  }

  // render light color border behind island
  public void renderBeach() {
    noStroke();
    PVector pos = this.getRelativePos();
    fill(253 * island_lum, 251 * island_lum, 249 * island_lum);
    circle(pos.x, pos.y, this.getSize());
  }

  // render all palmtrees belonging to island
  public void renderPalmtrees() {
    for (Palmtree palmtree : this.palmtrees) {
      palmtree.render();
    }
  }
}

// generate islands in radius
public ArrayList<Island> generate_islands(int numIslands, float radius) {
  ArrayList<Island> islands = new ArrayList<Island>();
  for (int i =0; i < numIslands; i++) {
    Island newIsland = new Island(new PVector(0, 0), min(radius, 200 * i));
    newIsland.insertPalmtrees();
    islands.add(newIsland);
  }
  return islands;
}

// render island water
public void render_islandWater(ArrayList<Island> islands) {
  for (Island island : islands) {
    island.renderIslandWater();
  }
}

// renders island background
public void render_islandBackground(ArrayList<Island> islands) {
  for (Island island : islands) {
    island.renderBeach();
  }
}

// renders all islands + palmtrees
public void render_islands(ArrayList<Island> islands) {
  for (Island island : islands) {
    island.render();
  }

  // render palmtrees after all islands have been rendered
  for (Island island : islands) {
    island.renderPalmtrees();
  }
}

public boolean onWater () {
  for (Island island : islands) {
    if (player.touching(island)) {
      return false;
    }
  }
  return true;
}
public class Palmtree extends Plottable {
  public float size;
  public PImage palmtreeImage;
  
  Palmtree(PVector pos, float radius) {
    super(pos, radius);
    this.size = 40;
  }

  // access 
  public float getSize() {
    return this.size;
  }
  
  // render palmtree
  public void render() {
    PImage img = this.palmtreeImage;
    if (img != null){
      PVector pos = this.getRelativePos();
      image(img, pos.x, pos.y);
    } else {
    noStroke();
    fill(0, 255, 0);
    circle(this.getPosition().x, this.getPosition().y, size);
    }
  }
}
Player player = new Player(new PVector(0, 0));
SoundFile playerDeathSound;
SoundFile destroyPalmtree;

public class Player extends Plottable {

  private float size;
  public int life;
  public SoundFile deathSound;

  Player(PVector pos) {
    super(pos);
    this.size = 15;
    this.life = 3;
  }

  public float getSize() {
    return this.size;
  }

  public void update() {
    float terrainMulti = 0.02f;
    if (onWater()) {
      terrainMulti = 0.01f;
    }

    PVector movement = new PVector(mouseX - center.x, mouseY - center.y);
    float magnitude = constrain(map(movement.mag(), 20, 150, 0, 100), 0, 100);
    PVector direction = movement.normalize().mult(magnitude * terrainMulti);
    this.move(direction);
  }

  public void attacked() {
    if (onWater()) {
      for (Hostile hostile : hostiles) {
        if (player.touching(hostile)) {
          println("Player attacked by ", hostile.type);
          hostile.playSound();
          this.playerDeath();
        }
      }
    }
  }
  
  public void playerDeath() {
    playerDeathSound.play();
    println("PLAYER DIED");
    PROGRAM_STATE = 2; 
  }
  
  public void playerWin(){
    if (wood.count >= 30 && bottles.count >= 15){
      PROGRAM_STATE = 3;
    }
  }

  public void displayInventory(float x, float y) {
    textSize(16);
    fill(0);
    text("Wood: " + wood.count, x, y);
    text("Bottles: " + bottles.count, x, y - 20);
  }

  public void render() {
    stroke(0);
    fill(255, 0, 0);
    circle(center.x, center.y, this.size);
  }
}
public abstract class Plottable {

  private PVector pos;

  // Constructors

  //
  // Create a Plottable at a given location
  //
  Plottable (PVector pos) {
    this.pos = pos.copy();
  }

  //
  // Create a Plottable within a radius of a given location
  //
  Plottable (PVector pos, float radius) {
    float distance = random(-radius, radius);
    PVector offset = PVector.random2D().mult(distance);
    this.pos = PVector.add(pos, offset);
  }

  // Access Functions

  public abstract float getSize ();

  public PVector getPosition () {
    return this.pos.copy();
  }

  public PVector getRelativePos () {
    PVector delta = player.getPosition().sub(this.pos);
    return center.copy().sub(delta);
  }

  public float distTo (Plottable other) {
    return this.pos.dist(other.getPosition());
  }

  public boolean touching (Plottable other) {
    float larger = 0.5f * max(this.getSize(), other.getSize());
    if (this.distTo(other) <= larger) {
      return true;
    }
    return false;
  }

  public boolean visible () {
    if (this.distTo(player) < (width / 2) + (height / 4) + this.getSize()) {
      return true;
    }
    return false;
  }

  // Modifier Functions

  public void move (PVector delta) {
    this.pos.add(delta);
  }

  // Processing

  public abstract void render ();
}

public ArrayList<Background> tiles;

public class Background extends Plottable {
  
  Background (PVector pos) {
    super(pos);
    tiles.add(this);
  }
  
  public float getSize () {
    return 1000;
  }
  
  public void update () {
    if (frameCount % 4 == 0) {
      this.move(new PVector(1, 1));
    }
    if (super.pos.x >= 3064) {
      super.pos.x = -3064;
    }
    if (super.pos.y >= 3064) {
      super.pos.y = -3064;
    }
  }
  
  public void render () {
    if (this.visible()) {
      tint(255, 80);
      PVector local = this.getRelativePos();
      image(background_image, local.x, local.y);
      tint(255, 255);
    }
  }
}

public void create_background () {
  tiles = new ArrayList<Background>();
  for (int i = -2560; i <= 2560; i += 1024) {
    for (int j = -2560; j <= 2560; j += 1024) {
      new Background(new PVector(i, j)).update();
    }
  } 
}

public void handle_background () {
  for (Background b : tiles) {
    b.update();
    b.render();
  }
}
  
  
ArrayList<Resource> resources = new ArrayList<Resource>();

public class Resource {
  
  private String name;
  private int count;
  private PImage loadedImage;
  private ArrayList<ResourceInstance> instances;
  
  Resource (String name) {
    this.name = name;
    this.count = 0;
    this.instances = new ArrayList<ResourceInstance>();
  }
  
  Resource (String name, String imagePath) {
    this.name = name;
    this.count = 0;
    this.loadedImage = loadImage(imagePath);
    this.instances = new ArrayList<ResourceInstance>();
  }
  
  Resource (String name, String imagePath, String type) {
    this.name = name;
    this.count = 0;
    this.loadedImage = loadImage(imagePath, type);
    this.instances = new ArrayList<ResourceInstance>();
  }
  
  public String getName () {
    return this.name;
  }
  
  public int getCount () {
    return this.count;
  }
  
  public PImage getImage () {
    return this.loadedImage;
  }
  
  public ArrayList<ResourceInstance> getInstances () {
    return this.instances;
  }
  
  // Modifier Functions
  
  public void collect (int count) {
    this.count += count;
    pickup.play();
    println("collect called");
  }
  
  public void addInstance (ResourceInstance instance) {
    this.instances.add(instance);
  }
  
  // Processing
  public void renderInstances () {
    for (ResourceInstance inst : this.instances) {
      if (inst.visible()) {
        inst.render();
      }
    }
  }
  
}

public abstract class ResourceInstance extends Plottable {
  
  private Resource parent;
  
  public ResourceInstance (Resource type, PVector pos) {
    super(pos);
    this.parent = type;
    this.parent.addInstance(this);
  }
  
  public ResourceInstance (Resource type, PVector pos, float radius) {
    super(pos, radius);
    this.parent = type;
    this.parent.addInstance(this);
  }
  
  //
  // Default rendering in case Resource Instance does not override
  //
  public void render () {
    PImage img = this.parent.getImage();
    PVector pos = this.getRelativePos();
    float size = this.getSize();
    if (img != null) {
      image(img, pos.x, pos.y, size, size);
    } else {
      fill(255);
      stroke(0);
      circle(pos.x, pos.y, size);
    }
  }
  
}

public void renderResources() {
  for (Resource resource : resources) {
    resource.renderInstances();
  }
}
int value;
public void keyPressed() {

  if (key == 'r') {
    settings();
  }
  
  if (key == 'p') {
    PROGRAM_STATE = 2;
  }
  
  // Interact with item
  if (key == 'f') {
    
    // Destorying a palmtree
    for (Island island : islands) {
      if (player.touching(island)) {
        for (int i = island.palmtrees.size()-1; i >= 0; i--) { // reverse loop to remove palmtree while iterating
          if (player.touching(island.palmtrees.get(i))) {
            WoodInstance woodPiece = new WoodInstance(island.palmtrees.get(i).getPosition(), 5);
            island.palmtrees.remove(i);
            destroyPalmtree.play();
            println("player destroyed a palmtree");
            break;
          }
        }
      }
    }
  }
  
  // enter game from menu screen
  if (key == 'e' && PROGRAM_STATE == 0){
    PROGRAM_STATE = 1;
  }
  
  // Collect Item
  if (key == 'e') {
    // Collecting logs
    for (int i = wood.instances.size() - 1; i >= 0; i--) {
      if (player.touching(wood.instances.get(i))) {
        wood.collect(1);
        wood.instances.remove(i);
        println("total wood count: ", wood.count);
        break;
      }
    }
    
    // Collecting Bottles
    for (int i = bottleArray.size() - 1; i >= 0; i--) {
      if (player.touching(bottleArray.get(i))) {
        bottles.collect(1);
        bottleArray.remove(i);
        println("player touched a bottle");
        break;
      }
    }
  }

  if (key == 'q' && PROGRAM_STATE > 1){
    exit();
  }
}
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "misplaced" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
